@@ -1218,6 +1218,8 @@ def running(name,
           ``--net=none``)
         - ``container:<name_or_id>`` - Reuses another container's network stack
         - ``host`` - Use the host's network stack inside the container
+        - Any name that identifies an existing network that might be created
+          with ``dockerng.network_present``.
 
           .. warning::
 
@@ -1967,7 +1969,7 @@ def network_present(name, driver=None, containers=None):
     Ensure that a network is present.
 
     name
-        Name of the netwotk
+        Name of the network
 
     driver
         Type of driver for that network.
@@ -2043,7 +2045,7 @@ def network_absent(name, driver=None):
     Ensure that a network is absent.
 
     name
-        Name of the netwotk
+        Name of the network
 
     Usage Examples:
 
@@ -2079,6 +2081,132 @@ def network_absent(name, driver=None):
     return ret
 
 
+def volume_present(name, driver=None, driver_opts=None):
+    '''
+    Ensure that a volume is present.
+
+    .. versionadded:: 2015.8.4
+
+    name
+        Name of the volume
+
+    driver
+        Type of driver for that volume.
+
+    driver_opts
+        Option for tha volume driver
+
+    Usage Examples:
+
+    .. code-block:: yaml
+
+        volume_foo:
+          dockerng.volume_present
+
+
+    .. code-block:: yaml
+
+        volume_bar:
+          dockerng.volume_present
+            - name: bar
+            - driver: local
+            - driver_opts:
+                foo: bar
+
+    .. code-block:: yaml
+
+        volume_bar:
+          dockerng.volume_present
+            - name: bar
+            - driver: local
+            - driver_opts:
+                - foo: bar
+                - option: value
+
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'result': False,
+           'comment': ''}
+    if salt.utils.is_dictlist(driver_opts):
+        driver_opts = salt.utils.repack_dictlist(driver_opts)
+    volumes = [v for v in __salt__['dockerng.volumes']()['Volumes'] if v['Name'] == name]
+    if not volumes:
+        try:
+            ret['changes']['created'] = __salt__['dockerng.create_volume'](
+                name, driver=driver, driver_opts=driver_opts)
+        except Exception as exc:
+            ret['comment'] = ('Failed to create volume \'{0}\': {1}'
+                              .format(name, exc))
+            return ret
+        else:
+            result = True
+            ret['result'] = result
+            return ret
+    # volume exits, check if driver is the same.
+    volume = volumes[0]
+    if volume['Driver'] != driver:
+        try:
+            ret['changes']['removed'] = __salt__['dockerng.remove_volume'](name)
+        except Exception as exc:
+            ret['comment'] = ('Failed to remove volume \'{0}\': {1}'
+                              .format(name, exc))
+            return ret
+        else:
+            try:
+                ret['changes']['created'] = __salt__['dockerng.create_volume'](
+                    name, driver=driver, driver_opts=driver_opts)
+            except Exception as exc:
+                ret['comment'] = ('Failed to create volume \'{0}\': {1}'
+                                .format(name, exc))
+                return ret
+            else:
+                result = True
+                ret['result'] = result
+                return ret
+
+    ret['result'] = True
+    ret['comment'] = 'Volume \'{0}\' already exists.'.format(name)
+    return ret
+
+
+def volume_absent(name, driver=None):
+    '''
+    Ensure that a volume is absent.
+
+    .. versionadded:: 2015.8.4
+
+    name
+        Name of the volume
+
+    Usage Examples:
+
+    .. code-block:: yaml
+
+        volume_foo:
+          dockerng.volume_absent
+
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'result': False,
+           'comment': ''}
+
+    volumes = [v for v in __salt__['dockerng.volumes']()['Volumes'] if v['Name'] == name]
+    if not volumes:
+        ret['result'] = True
+        ret['comment'] = 'Volume \'{0}\' already absent'.format(name)
+        return ret
+
+    try:
+        ret['changes']['removed'] = __salt__['dockerng.remove_volume'](name)
+        ret['result'] = True
+    except Exception as exc:
+        ret['comment'] = ('Failed to remove volume \'{0}\': {1}'
+                          .format(name, exc))
+    return ret
+
+
 def mod_watch(name, sfun=None, **kwargs):
     if sfun == 'running':
         watch_kwargs = copy.deepcopy(kwargs)
